#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <getopt.h>


#include "codepage.h"
#include "dbf.h"

char *conv;

int output_c (struct DBF_FIELD *field, unsigned char *buffer);
int output_n (struct DBF_FIELD *field, unsigned char *buffer);
int output_l (struct DBF_FIELD *field, unsigned char *buffer);
int output_d (struct DBF_FIELD *field, unsigned char *buffer);
int output_prerecord ();
int output_postrecord ();
int output_fieldsep (struct DBF_FIELD *field);
int output_fielddef(struct DBF_FIELD *field);
int output_fielddefsep(struct DBF_FIELD *field);
int output_dbdef(char *dbname);
int output_dbenddef();
int output_postdb ();

char *usage = "%s: [OPTIONS] [FILE1 [FILE2 [...]]]\n"
"\n"
"OPTIONS:\n"
"  --windows-1250       -w   DBF file is in Windows-1250 codepage\n"
"  --iso-8859-2         -i   DBF file is in ISO-8859-2 codepage\n"
"  --kamenic            -k   DBF file is in Kamenic (CP895) codepage\n"
"  --latin2             -l   DBF file is in PC Latin2 (CP852) codepage\n"
"  --koi8cs             -o   DBF file is in ancient KOI8CS codepage\n"
"\n"
"Report bugs to mhi@penguin.cz\n";

int
convertdb (char *dbname)
{
  FILE *dbf;
  struct DBF_HEADER hdr;
  int x;
  int nfields, maxlen;
  struct DBF_FIELD *fields;
  char *buffer, *number;
  unsigned int y;
  char c;

  if ((dbf = fopen (dbname, "rb")) == 0)
    {
      perror (dbname);
      return 0;
    }

  fread (&hdr, sizeof (hdr), 1, dbf);
  if (hdr.version != 0x03 && hdr.version&0x7F != 0x05 && hdr.version != 0xF5 
      && hdr.version != 0x83)
    {
      fprintf (stderr, "Only dBASE III is supported. This is version ");
      for (x = 0; x < hdr.version & 0x7f; x++)
	fprintf (stderr, "I");
      if (hdr.version & 0x80)
	fprintf (stderr, " with DBT");
      fprintf (stderr, ".\n");
      fclose (dbf);
      return 0;
    }

  nfields = (hdr.hdrlen - 33) / 32;

  fields = (struct DBF_FIELD *) malloc (nfields * sizeof (struct DBF_FIELD));
  if (!fields)
    {
      printf ("Not enough memory.");
      fclose (dbf);
      return 0;
    }

  fread (fields, sizeof (struct DBF_FIELD), nfields, dbf);
  maxlen = 0;


  output_dbdef(dbname);
  for (x = 0; x < nfields; x++)
    {
      if (maxlen < fields[x].fieldlen)
	maxlen = fields[x].fieldlen;

      output_fielddef(&fields[x]);

      if (x < nfields - 1)
        output_fielddefsep(&fields[x]);
    }
  output_dbenddef();

  fseek (dbf, 1, SEEK_CUR);
  maxlen++;

  buffer = (char *) malloc (maxlen);
  if (!buffer)
    {
      printf ("Not enough memory.");
      fclose (dbf);
      return 0;
    }

  for (y = 0; y < hdr.records; y++)
    {
      fseek (dbf, 1, SEEK_CUR);

      output_prerecord ();
      for (x = 0; x < nfields; x++)
	{
	  fread (buffer, fields[x].fieldlen, 1, dbf);
	  buffer[fields[x].fieldlen] = 0;
	  while (buffer[strlen (buffer) - 1] == ' ')
	    buffer[strlen (buffer) - 1] = 0;
	  number = buffer;
	  while (number[0] == ' ')
	    number++;
	  switch (fields[x].datatype)
	    {
	    case 'C':
	      output_c (&fields[x], buffer);
	      break;
	    case 'N':
	      output_n (&fields[x], buffer);
	      break;
	    case 'D':
	      output_d (&fields[x], buffer);
	      break;
	    case 'L':
	      output_n (&fields[x], buffer);
	      break;
	    case 'M':
              break;
	    }
	  if (x < nfields - 1)
	    output_fieldsep (&fields[x]);
	}
      output_postrecord ();
    }

  output_postdb();

  free (fields);
  free (buffer);
  fclose (dbf);
}

static struct option long_options[] =
{                               /* For parsing long arguments */
  {"windows-1250", no_argument, 0, 'w'},
  {"iso-8859-2", no_argument, 0, 'i'},
  {"koi8cs", no_argument, 0, 'o'},
  {"latin2", no_argument, 0, 'l'},
  {"kamenic", no_argument, 0, 'k'},
  {"version", no_argument, 0, 'v'},
  {"help", no_argument, 0, 'h'},
  {0, 0, 0, 0},
};
int option_index;               /* Unused */

int
main (int argc, char **argv)
{
  char c;
  char *argv0;

  argv0 = argv[0];

  conv = NULL;
  /* First we parse all the command line arguments */
  while ((c = getopt_long (argc, argv, "wiokvhl",
                           long_options, &option_index)) != -1)
    {
      switch (c)
        {
        case 'w':
            conv = win1250_conv;
          break;

        case 'k':
            conv = kamenic_conv;
          break;

        case 'l':
            conv = latin2_conv;
          break;

        case 'o':
            conv = koi8cs_conv;
          break;

 
        case 'i':
            conv = NULL;
          break;

        case 'v':
        case 'h':
          fprintf (stderr, usage, argv0);
          exit (0);
        default:
          fprintf (stderr, usage, argv0);
          exit (1);
        }
    }

  argc -= optind;
  argv += optind;

  if (argc == 0) {
          fprintf (stderr, usage, argv0);
          exit (1);
  }

  while (argc--)
    {
      convertdb (argv[0]);
      argv++;
    }
}
